NAME        TAIL  
PAGE  59,132
TITLE       'TAIL.ASM -- command line interaction for BAT files' 
  
;==========================================================================
;==								         ==
;==			            TAIL            		         ==
;==								         ==
;==      Created:   January 26, 1990				         ==
;==	 BY:        C. TEASLEY                                           ==      
;==      purpose:  Permits command line interaction in BAT files by      ==
;==                1. Printing PAR1 in "TAIL PAR1",then                  ==
;==                2. Pausing to accept user input, then                 ==
;==                3. Executing PAR1 with user supplied PAR2,PAR3,etc.   ==
;==      adapted from: Ray Duncan's Shell.asm from 'Advanced MS-DOS'     ==         
;==      used with:  1. "AUTOMAXX"; a shareware menu program             ==
;==                  2. Any batch file ( *.bat )                         ==
;==      To assemble and link to tail.exe:                               ==
;==                                                                      ==
;==                  E:MASM D:tail                                       ==              
;==                  E:LINK D:tail                                       ==
;==                                                                      ==                             
;==========================================================================
stdin       equ      0                          ;Standard input device
stdout      equ      1                          ;Standard output device
stderr      equ      2                          ;Standard error device
cr          equ      0dh                        ;ASCII carriage return
lf          equ      0ah                        ;ASCII line feed
blank       equ      20h                        ;ASCII blank code
esc         equ      01bh                       ;ASCII escape code
def_buf         equ     80h                     ;offset of default buffer
  
seg_a		segment para public 'CODE'      
		assume	cs:seg_a, ds:data,ss:stack
  
tail        	proc	far                     ;at entry DS=ES=PSP
                mov     ax,data                 ;let ds point to data segment
                mov     ds,ax  
                mov     [PSP],ES                ;store ES location 
                mov     ax,es:[002ch]           ;get segment of environment blk
                mov     env_seg,ax              ;from PSP and save it.
                                                ;now release unneeded memory
                mov     bx,100h                 ;ES = segment PSP,BX = 
                                                ;paragraphs needed
                mov     ah,4Ah                  ;Fn.4Ah=modify memory block
                int     21h                     ;transfer to DOS
                jnc     tail1                   ;jump if request successful
                mov     dx,offset msg1          ;otherwise display error msg
                mov     cx,msg1_length          ;and exit.
                jmp     tail4
tail1:          call    get_comspec             ;get file spec for command.com
                jnc     tail2                   ;jmp if found OK
                mov     dx,offset msg3          ;COMSPEC variable not found in
                mov     cx,msg3_length          ;Environment block,print error
                jmp     tail4                   ;message and exit
tail2:          mov     dx,offset tail3         ;set ctrl-C vector (Int 23h)
                mov     ax,cs                   ;so that "tail" keeps control
                mov     dx,ax                   ;ds:dx = address of ^C handler
                mov     ax,2523h                ;Fn. 25= set interrupt
                int     21h
                mov     ax,data                 ;make ds & es point to data
                mov     ds,ax                   ;segment again
                mov     es,ax
tail3:                                          ;main loop of tail program
                ;call    cls_cmd                 ;clear screen with lf's
                call    cls_cmd1                ;clear screen w/int10/0
                call    get_prompt              ;get and print pgm_name
                                                ;store it in "prompt"
                call    get_cmd                 ;get cmd tail from user
                call    extrinsic               ;pass program name & tail
                                                ;to Command.com
                call    exit_cmd                ;normal exit
tail4:                                          ;come here if error detected
                                                ;with ds:dx=msg,cx=length
                mov     bx,stderr               ;print msg on stderr device
                mov     ah,40h
                int     21h
                mov     ax,4c01h                ;exit to DOS with ret. code=1
                int     21h                     ;indicating an error condition
tail            endp
                                                ;these vars reqd in code seg:
stk_seg         dw      0                       ;original SS contents
stk_ptr         dw      0                       ;original SP contents

get_prompt      proc    near                    ;read 0000:080h (def_buf)
                push    ds                      ;save ds segment location        
                mov     ax,[psp]
                mov     ds,ax                   ;segment of 080h
                mov     ax,080h+2               ;offset of def_buf +2
                mov     si,ax
                mov     di,offset inp_buf       ;point to inp_buf
                mov     al,ds:[080h]            ;number of bytes to move
                cbw                             ;zero ah
                mov     cx,ax
                mov     ES:byte ptr [name_length],al  ;store length of cmd name
                cld
                rep     movsb                   ;program name w/o cr in inp_buf
                pop     ds                      ;restore segment location
                mov     byte ptr[di-1],20h      ;store " " after name
print_name:     mov     bx,stdout               ;print name as prompt
                mov     dx,offset inp_buf       ;address to be printed
                mov     al,byte ptr[name_length]  ;restore name length
                inc     al                      ;+ " "
                cbw
                mov     cx,ax
                mov     ah,40h                  ;on std output
                int     21h
                ret
get_prompt      endp

get_cmd         proc    near                    ;Get a cmd from user.
                mov     dx,offset inp_buf
                mov     al,byte ptr[name_length]
                cbw
                push    ax
                add     dx,ax                   ;enter after name+" "
                mov     cx,inp_buf_length
                pop     ax
                sub     cx,ax                   ;space left in buffer
                mov     bx,stdin                ;Place cmd line in input buffer
                mov     ah,3fh                  ;Fn 3F read file or device
                int     21h
                mov     si,offset inp_buf       ;Fold all lowercase characters
                mov     cx,inp_buf_length       ;in cmd line to uppercase
gcmd1:          cmp     byte ptr[si],'a'
                jb      gcmd2
                cmp     byte ptr[si],'z'
                ja      gcmd2
                sub     byte ptr[si],'a'-'A'
gcmd2:          inc     si
                loop    gcmd1
                ret                             ;back to caller
get_cmd         endp

extrinsic       proc    near                    ;process user's parameters by
                                                ;adding them to inp_buf and 
                                                ;passing entire buffer to 
                mov     al,cr                   ;cmd.com.
                mov     cx,cmd_tail_length      ;Find length of cmd by scanning
                mov     di,offset cmd_tail+1    ;for cr.
                cld
                repnz   scasb
                mov     ax,di
                sub     ax,offset cmd_tail+2    ;calc length w/o cr
                mov     byte ptr[cmd_tail],al   ;store length for exec funct.
                mov     par_cmd,offset cmd_tail ;address of cmd tail
                call    exec                    ;Call exec funct to pass
                ret                             ;cmd line to COMMAND.COM
extrinsic       endp
get_comspec     proc    near                    ;get file spec of COMMAND.COM
                                                ;from Environment.
                mov     si,offset com_var
                call    get_env
                jc      gcsp2
                mov     si,offset com_spec
gcsp1:          mov     al,es:[di]
                mov     [si],al
                inc     si
                inc      di
                or      al,al
                jnz     gcsp1
gcsp2:          ret
get_comspec     endp

get_env         proc    near
                mov     es,env_seg
                xor     di,di
genv1:          mov     bx,si
                cmp     byte ptr es:[di],0
                jne     genv2
                stc
                ret
genv2:          mov     al,[bx]
                or      al,al
                jz      genv3
                cmp     al,es:[di]
                jne     genv4
                inc     bx
                inc     di
                jmp     genv2
genv3:          ret
genv4:          xor     al,al
                mov     cx,-1
                cld
                repnz   scasb
                jmp     genv1
get_env         endp

exec            proc    near
                push    ds
                push    es
                mov     cs:stk_seg,ss
                mov     cs:stk_ptr,sp
                mov     dx,offset com_spec
                mov     bx,offset par_blk
                mov     ah,4bh
                mov     al,0                    ;subfunc 0=load and execute
                int     21h
                mov     ss,cs:stk_seg
                mov     sp,cs:stk_ptr
                pop     es
                pop     ds
                jnc     exec1
                mov     dx,offset msg2
                mov     cx,msg2_length
                mov     ah,40h
                int     21h
exec1:          ret
exec            endp

cls_cmd         proc    near
                mov    dx,offset cls_str
                mov    cx,cls_str_length
                mov     bx,stdout
                mov     ah,40h 
                int     21h
                ret
cls_cmd         endp

cls_cmd1        proc    near                    ;int 10,fn 0
                mov     ax,07h
                int     10h
                ret
cls_cmd1        endp
                
dos_cmd         proc    near
dos_cmd         endp
exit_cmd        proc    near
                mov     ax,4c00h
                int     21h
exit_cmd        endp

seg_a           ends

stack           segment para stack 'STACK'
                dw      64 dup (?)
stack           ends
                
data            segment para public 'DATA'

com_var         db      'COMSPEC=',0
com_spec        db      80 dup (0)
name_length     db      0,0
cmd_tail        db      0,' /C '
inp_buf         db      80 dup (0)
inp_buf_length  equ     $-inp_buf
cmd_tail_length equ     $-cmd_tail-1
PSP             dw      0
env_seg         dw      0
msg1            db      cr,lf
                db      'Unable to de-allocate memory.'
                db      cr,lf
msg1_length     equ     $-msg1
msg2            db      cr,lf
                db      'EXEC of COMMAND.COM failed.'
                db      cr,lf
msg2_length     equ     $-msg2
msg3            db      cr,lf
                db      'No COMSPEC variable in Environment.'
                db      cr,lf
msg3_length     equ     $-msg3
cls_str         db      25h dup (lf)            ;form feed
cls_str_length  equ     $-cls_str
par_blk         equ     $
                dw      0
par_cmd         dw      offset cmd_tail
                dw      seg cmd_tail
                dd      -1
                dd      -1

data            ends

                end             tail                             


